#pragma once
#include <fstream>
#include <set>
#include "Exception.h"
#include <iostream>

class TravelCompany {
private:
	struct Data {
		std::string tour_name;
		std::string surname;
		float price;
		int days;
		float moving_cost;
		float exchange_rate;
		float amount;
		float travel_cost;

		Data() :
			tour_name(""),
			surname(""),
			price(0.0),
			days(0),
			moving_cost(0.0),
			exchange_rate(0.0),
			amount(0.0),
			travel_cost(0.0)
		{}

		Data(
			const std::string& tour_name,
			const std::string& surname,
			float price,
			int days,
			float moving_cost,
			float exchange_rate,
			float amount,
			float travel_cost) :
			tour_name(tour_name),
			surname(surname),
			price(price),
			days(days),
			moving_cost(moving_cost),
			exchange_rate(exchange_rate),
			amount(amount),
			travel_cost(travel_cost)
		{}

		bool operator == (const Data& data) {
			return (this->tour_name == data.tour_name &&
				this->surname == data.surname &&
				this->price == data.price &&
				this->days == data.days &&
				this->moving_cost == data.moving_cost &&
				this->exchange_rate == data.exchange_rate &&
				this->amount == data.amount &&
				this->travel_cost == data.travel_cost);
		}
		bool operator != (const Data& data) {
			return !(*this == data);
		}

		friend bool operator < (const Data& data, const Data& data2) {
			return data.travel_cost < data2.travel_cost;
		}

		bool in(const std::multiset<Data> cont) {
			for (auto i = cont.begin(); i != cont.end(); ++i) {
				if (*this == *i)
					return true;
			}
			return false;
		}

	};

	std::multiset<Data> cont;

public:
	TravelCompany() : cont() {}
	TravelCompany(std::multiset<Data> cont) : cont(cont) {}

	bool operator == (const TravelCompany&);

	bool operator != (const TravelCompany& data) {
		return !(*this == data);
	}

	friend std::ostream& operator << (std::ostream&, const TravelCompany&);
	friend std::istream& operator >> (std::istream&, TravelCompany&);

	void create_file(const std::string&);

	void add(const std::string&);
	void delete_data(const std::string&);
	void replacement(const std::string&, TravelCompany&);
	void transform_to_txt(const std::string& filepath) {
		if (rename((filepath + ".bin").c_str(), (filepath + ".txt").c_str()) != 0)
			throw FileException(2);
	}

};